# THIS IS EASYBFTUTORIAL.R IT USES AN LM OBJECT INPUT FOR
# BAIN TO EVALUATE (INFORMATIVE) HYPOTHESES
#
# THERE IS A COUNTERPART BFTUTORIAL.R WHICH USES A NAMED 
# VECTOR INPUT FOR BAIN TO EVALUATE (INFORMATIVE) HYPOTHESES
#
# EASY LITERALLY MEANS "MADE AS EASY AS POSSIBLE FOR THE USER"
# THE COUNTERPART SHOWS MORE OF THE STEPS INVOLVED IN 
# BAIN HYPOTHESES EVALUATION.

#***************
#TUTORIAL STEP 1
#***************
# Load libraries. These libraries contain functions such as lm and bain that will be used in 
# this R code. Each time you reopen EASYBFTutorial.R you have to execute this step.

library(MASS)
library(bain)
library(psych)


#***************
#TUTORIAL STEP 2
#***************
# Read Data. If you open monin.txt and holbar.txt in a text editor, you can see
# that variable labels have been inserted between " " in the first line of the file
# (this is called a header).

monin<-read.table("monin.txt",header=TRUE)
monin$group<-factor(monin$group)       # this command tells R that group is a factor and 
# not a continuous variable like attract
holubar<-read.table("holubar.txt",header=TRUE)
holubar$gr <- factor(holubar$gr)       # this command tells R that gr is a factor and 
# not a continuous variable like at

#***************
#TUTORIAL STEP 3
#***************
# Compute descriptives for the Monin data. The dependent variable name is attract, 
# the factor name is group (see also TutorialStep 2).

descrip <- describeBy(monin$attract,monin$group,mat=TRUE)
print(descrip)


#***************
#TUTORIAL STEP 4
#***************
# Compute the Bayes factor for the Monin data. 
# If you return to BFTutorial.R after closing it, you always have to rerun 
# Steps 1 and 2

# Using the R package lm the means of attract in each of the groups and the residual variance are estimated
# for the Monin data. Note that, attract~group-1 intstructs lm to regress attract on group. The -1 instructs
# lm to estimate the means in each group. Without the -1 regular dummy coding would have been applied. The
# results are collected in, what is called, the R object, prepmonin
prepmonin <-  lm(attract~group-1, data=monin)

# Inspect the estimates of the means and their names 
coef(prepmonin)

# Run Bain with the information available in prepmonin.
#
# The null-hypothesis that the three means are equal is provided via 
# "group1 = group2 = group3". See the bain help file for the requirements
# that hypotheses have to adhere to. Note that, the names used in the
# hypothesis can be seen using "coef(prepmonin).
#
# To be able to repeat an analysis exactly, always set the seed before running bain.
# It is recommended to run analyses with two different seeds to ensure stability of
# the results.
#
# The results of the analysis are stored in the R object resmonin.

set.seed(100)
resmonin<-bain(prepmonin,"group1 = group2 = group3")

# print the results of the analysis with bain
print(resmonin)

#***************
#TUTORIAL STEP 5
#***************
# Compute the Bayes factor for "all" the hypotheses for the Monin data
# Note that prepmonin has been obtained in TUTORIAL STEP 4.
set.seed(100)
resmonin<-bain(prepmonin,"group1 = group2 = group3; group1 = group2; 
               group1 = group3; group2 = group3")

# print the results of the analysis with bain

print(resmonin)

#***************
#TUTORIAL STEP 6
#***************
# Bayesian updating. Stepwise increasing the sample size for the Monin data

# WARNING: INCREASE THE SIZE OF THE PLOT WINDOW IN THE LOWER RIGHT HAND SCREEN
# OTHERWISE YOU ARE BOUND TO RECEIVE AN ERROR MESSAGE

# create an R object BFmonin in which the Bayes factor for different sample sizes 
# will be collected.
BFmonin<-1

# start with an initial sample size of 2+0=2 persons per group an incease by 1 
# until 2+17=19 persons per group

for (i in 0:17){

# collect the subset of persons selected in a matrix  
subdata <- matrix(0, nrow = 6+3*i, ncol = 2)  # in total 6 initial persons in 3 groups
                                              # for which 2 variables have been recorded
                                              # when i=0, each group contains 2 persons
                                              # when i=1, 1 additional person is added to each of the 3 groups
                                              # when 1=17, 17 additional persons are added to each group
subdata <- monin[c(1:(2+i),20:(21+i),39:(40+i)),1:2] # the first person in the first group is 1
                                                     # the first person in the second group is 20
                                                     # the first person in the third group is 39
                                                     # when i=0 each group contains 2 persons
                                                     # when i=5 each group contains 2+5=7 persons

submonin <-  lm(attract~group-1, data=subdata)

set.seed(100)
resmonin<-bain(submonin,"group1 = group2 = group3")

# collect BF0a from the object resmonin and store in BFmonin. This is done for i=1 to 17.
BFmonin[1+i]<-resmonin$fit$BF[1]
}

# create a vector containing the sample size per group
NperGroup<-c(2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19)
# plot NperGroup vs BFmonin, add labels for the y-axis and the x-axis
plot(NperGroup,BFmonin,ylab="BF0a",xlab="N per Group")
# and connect the points by a line
lines(NperGroup,BFmonin)

#***************
#TUTORIAL STEP 7
#***************
# Bayesian updating. Posterior Model Probabilities for the Monin Data

# Create a vector in which the 5 PMPs resulting from Bain can be stored
pall<-1
# Create a vector in which each of the 5 PMPs resulting from Bain analysis with increasing sample
# sizes can be stored
p1<-1
p2<-1
p3<-1
p4<-1
p5<-1

for (i in 0:17){
  
  subdata <- matrix(0, nrow = 6+3*i, ncol = 2)
  subdata <- monin[c(1:(2+i),20:(21+i),39:(40+i)),1:2]
  
  submonin <-  lm(attract~group-1, data=subdata)

  set.seed(100)
  resmonin<-bain(submonin,"group1 = group2 = group3; group1 = group2; 
                 group1 = group3; group2 = group3")
  
# The posterior model probabilities obtained for the sample size at hand are collected from the 
# R object resmonin which contains the results of the Bain analysis
  pall <- resmonin$fit$PMPb
# # R thinks that the numbers stored in pall are labels. The command below tells it otherwise.
#   pall <- as.numeric(levels(pall))[pall]

# Story the PMPs for each hypothesis in a vector, that is, collect the PMPs obtained for the different
# sample sizes in a vector
  p1[i+1]<-pall[1]
  p2[i+1]<-pall[2]
  p3[i+1]<-pall[3]
  p4[i+1]<-pall[4]
  p5[i+1]<-pall[5]
}
NperGroup<-c(2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19)

# create a legenda for the plot
leg <- c("H0","Ha1","Ha2","Ha3","Ha")

# give each line a different appearance
lty <- c(1,2,3,4,5)

# plot NperGRoup versus p1 using a line plot (type = "l") and a y-axis bounded between 0 and 1 (ylim=(0:1.0))
plot(NperGroup,p1,type = "l",ylab = "posterior probabilities",xlab="N per Group",ylim=(0:1.0))
# add a line for p2, p3, p4, and p5 to the plot
lines(NperGroup,p2,lty=2)
lines(NperGroup,p3,lty=3)
lines(NperGroup,p4,lty=4)
lines(NperGroup,p5,lty=5)
# insert the legenda in the plot at x=17 and y=.95 
legend(x = 17, y = 0.95,legend = leg, lty =lty)

#****************
#TUTORIAL STEP 8A
#****************
# Sensitivity Analysis Monin Data

prepmonin <-  lm(attract~group-1, data=monin)

# compute BF0a using b-g
set.seed(100)
resmonin<-bain(prepmonin,"group1 = group2 = group3", fraction = 1)
print(resmonin)

# compute BF0a replacing b-g by 2 x b-g
set.seed(100)
# setting fraction to 2 implies using 2 x b-g
resmonin<-bain(prepmonin,"group1 = group2 = group3", fraction = 2)
print(resmonin)

# compute BF0a replacing b-g by 3 x b-g
set.seed(100)
resmonin<-bain(prepmonin,"group1 = group2 = group3", fraction = 3)
print(resmonin)

#****************
#TUTORIAL STEP 8B
#****************
# Sensitivity Analysis Holubar Data

prepholubar <-  lm(at~gr-1, data=holubar)

# compute BF0a using b-g
set.seed(100)
resholubar<-bain(prepholubar,"gr1 = gr2 = gr3",fraction = 1)
print(resholubar)

# compute BF0a replacing b-g by 2 x b-g
set.seed(100)
resholubar<-bain(prepholubar,"gr1 = gr2 = gr3", fraction = 2)
print(resholubar)

# compute BF0a replacing b-g by 3 x b-g
set.seed(100)
resholubar<-bain(prepholubar,"gr1 = gr2 = gr3", fraction = 3)
print(resholubar)

#***************
#TUTORIAL STEP 9
#***************
# Compute the Bayes factor for the Monin data with two outliers

# Created for the third group are two persons with scores of 9 and 10 on attract.
# These have been added to monin.txt rendering the file moninoutlier.txt

monoutl<-read.table("moninoutlier.txt",header=TRUE)
monoutl$group <- factor(monoutl$group)

prepoutlier <-  lm(attract~group-1, data=monoutl)

set.seed(100)
resbain<-bain(prepoutlier,"group1 = group2 = group3")

# print the results of the analysis with bain

print(resbain)

#****************
#TUTORIAL STEP 10
#****************

prepmonin <-  lm(attract~group-1, data=monin)

# Evaluating informative hypotheses using teh Monin data
set.seed(100)
resmonin<-bain(prepmonin,"group1 > group2 > group3; 
group1 > group2 = group3; group1 = group2 & group1 > group3")

# print the results of the analysis with bain
print(resmonin)
# use the command below to obtain the BFmatrix
print(resmonin$BFmatrix)

#****************
#TUTORIAL STEP 11
#****************

prepmonin <-  lm(attract~group-1, data=monin)

# Sensitivity Analyis for the  Evaluation of Inequality Constrained Hypotheses
set.seed(100)
resmonin<-bain(prepmonin,"group1 > group2 > group3",fraction = 1)
print(resmonin)

set.seed(100)
resmonin<-bain(prepmonin,"group1 > group2 > group3",fraction = 2)
print(resmonin)

set.seed(100)
resmonin<-bain(prepmonin,"group1 > group2 > group3",fraction = 3)
print(resmonin) 

#*****************
#TUTORIAL STEP 12A
#*****************
# Replication of Monin using the Holubar data

# Compute descriptives the Holubar data. The dependent variable name is at,
# the factor name is gr (see also TutorialStep 2)

descrip <- describeBy(holubar$at,holubar$gr,mat=TRUE)
print(descrip)

#*****************
#TUTORIAL STEP 12B
#*****************
# Evaluate H0 and Ha using the Holubar data 
prepholubar <-  lm(at~gr-1, data=holubar)

set.seed(100)
resholubar<-bain(prepholubar,"gr1 = gr2 = gr3")
print(resholubar)

#*****************
#TUTORIAL STEP 12C
#*****************
# Evaluate Horiginal as derived from the Monin analysis Using the Holubar data

set.seed(100)
resholubar<-bain(prepholubar,"gr1 = gr2 = gr3; gr1 = gr2 & gr1 > gr3")
print(resholubar)

# To obtain the sample sizes, estimates and covariance matrix run the lines below
print(resholubar$n)
print(resholubar$estimates)
print(resholubar$posterior)

# ********************************************************************
# Below you find the codes used to recreate the monin and holubar data 
# ********************************************************************

# set the seed of the random number generator

# set.seed(123) # note that the seed used to recreate the monin and holubar data 
# used in the tutorial has been lost. This is irrelevant because the sufficient
# statistics of each data set (N, mean, sd) will be the same.

# Create the monin data

# Create variable attract
attract <- 1

# Generate normal random deviation for 67 persons (the Monin sample size)
attract[1:67]<-rnorm(67,mean=0,sd=1)

# For each of the three groups standardize the sampled values
attract[1:19]<-(attract[1:19]-mean(attract[1:19]))/sd(attract[1:19])
attract[20:38]<-(attract[20:38]-mean(attract[20:38]))/sd(attract[20:38])
attract[39:67]<-(attract[39:67]-mean(attract[39:67]))/sd(attract[39:67])

# For each group multiply with the standard deviation and add the mean as reported by Monin
attract[1:19]<-attract[1:19]*1.38+1.88
attract[20:38]<-attract[20:38]*1.95+2.54
attract[39:67]<-attract[39:67]*2.38+.02
group <- 1

# Create a factor group with the numbers 1, 2 and 3
group[1:19]<-1 #"ob"
group[20:38]<-2 #"rsa"
group[39:67]<-3 #"rbog"

# Create a data frame containing a header, attract and group and write it to a file monin.txt
monin<-matrix(c(attract,group),nrow=67,ncol=2)
monin<-as.data.frame(monin)
colnames(monin) <- c("attract", "group")
write.table(monin,"monin.txt",sep=" ",col.names=TRUE,row.names=FALSE)

# create the holubar data

at <- 1
at[1:75]<-rnorm(75,mean=0,sd=1)
at[1:20]<-(at[1:20]-mean(at[1:20]))/sd(at[1:20])
at[21:47]<-(at[21:47]-mean(at[21:47]))/sd(at[21:47])
at[48:75]<-(at[48:75]-mean(at[48:75]))/sd(at[48:75])
at[1:20]<-at[1:20]*1.20+.98
at[21:47]<-at[21:47]*1.88+.02
at[48:75]<-at[48:75]*1.72+.27
gr <- 1
gr[1:20]<-1 #"ob"
gr[21:47]<-2 #"rsa"
gr[48:75]<-3 #"rbog"
holubar<-matrix(c(at,gr),nrow=75,ncol=2)
holubar<-as.data.frame(holubar)
colnames(holubar) <- c("at","gr")
write.table(holubar,"holubar.txt",sep=" ",col.names=TRUE,row.names=FALSE)

